# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.2.0

import winrt.system
from winrt import _winrt_windows_phone_ui_input

__all__ = [
    "BackPressedEventArgs",
    "CameraEventArgs",
    "HardwareButtons",
]

BackPressedEventArgs = _winrt_windows_phone_ui_input.BackPressedEventArgs
CameraEventArgs = _winrt_windows_phone_ui_input.CameraEventArgs
HardwareButtons = _winrt_windows_phone_ui_input.HardwareButtons
